@@ -20,11 +20,8 @@
 
 """Module for implementing dialog box for viewing/editing data."""
 
-from __future__ import division
-
 import numpy as N
 
-from ..compat import cstr
 from .. import qtall as qt
 from .. import document
 from .. import datasets
@@ -148,8 +145,8 @@ class DatasetTableModel1D(qt.QAbstractTableModel):
         ops = document.OperationMultiple([], descr=_('set value'))
         if data is None:
             ops.addOperation(
-                document.OperationDatasetAddColumn(self.dsname,
-                                                   ds.columns[column]))
+                document.OperationDatasetAddColumn(
+                    self.dsname, ds.columns[column]))
 
         # add a row if necessary
         if row == len(ds.data):
@@ -163,9 +160,8 @@ class DatasetTableModel1D(qt.QAbstractTableModel):
             return False
 
         ops.addOperation(
-            document.OperationDatasetSetVal(self.dsname,
-                                            ds.columns[column],
-                                            row, val))
+            document.OperationDatasetSetVal(
+                self.dsname, ds.columns[column], row, val))
         try:
             self.document.applyOperation(ops)
         except RuntimeError:
@@ -414,8 +410,8 @@ class DatasetTableModel2D(qt.QAbstractTableModel):
                 len(self.yedge)-section-2]
             v2 = self.xedge[section+1] if xaxis else self.yedge[
                 len(self.yedge)-section-1]
-            return u'%s\u2013%s' % (setting.ui_floattostring(v1),
-                                    setting.ui_floattostring(v2))
+            return '%s\u2013%s' % (
+                setting.ui_floattostring(v1), setting.ui_floattostring(v2))
 
         return None
 
@@ -541,7 +537,7 @@ class ViewDelegate(qt.QStyledItemDelegate):
 
 class DataEditDialog(VeuszDialog):
     """Dialog for editing and rearranging data sets."""
-    
+
     def __init__(self, parent, document):
         VeuszDialog.__init__(self, parent, 'dataedit.ui')
         self.document = document
@@ -560,7 +556,7 @@ class DataEditDialog(VeuszDialog):
             (_('Copy'), self.slotCopy),
             (_('Delete row'), self.slotDeleteRow),
             (_('Insert row'), self.slotInsertRow),
-            ):
+        ):
             act = qt.QAction(text, self)
             act.triggered.connect(slot)
             self.datatableview.addAction(act)
@@ -587,20 +583,23 @@ class DataEditDialog(VeuszDialog):
             self.slotDatasetsSelected)
 
         # connect buttons
-        for btn, slot in ( (self.deletebutton, self.slotDatasetDelete),
-                           (self.unlinkbutton, self.slotDatasetUnlink),
-                           (self.duplicatebutton, self.slotDatasetDuplicate),
-                           (self.importbutton, self.slotDatasetImport),
-                           (self.createbutton, self.slotDatasetCreate),
-                           (self.editbutton, self.slotDatasetEdit),
-                           ):
+        for btn, slot in (
+                (self.deletebutton, self.slotDatasetDelete),
+                (self.unlinkbutton, self.slotDatasetUnlink),
+                (self.duplicatebutton, self.slotDatasetDuplicate),
+                (self.importbutton, self.slotDatasetImport),
+                (self.createbutton, self.slotDatasetCreate),
+                (self.editbutton, self.slotDatasetEdit),
+        ):
             btn.clicked.connect(slot)
 
         # menu for new button
         self.newmenu = qt.QMenu()
-        for text, slot in ( (_('Numerical dataset'), self.slotNewNumericalDataset),
-                            (_('Text dataset'), self.slotNewTextDataset),
-                            (_('Date/time dataset'), self.slotNewDateDataset) ):
+        for text, slot in (
+                (_('Numerical dataset'), self.slotNewNumericalDataset),
+                (_('Text dataset'), self.slotNewTextDataset),
+                (_('Date/time dataset'), self.slotNewDateDataset)
+        ):
             a = self.newmenu.addAction(text)
             a.triggered.connect(slot)
         self.newbutton.setMenu(self.newmenu)
@@ -628,7 +627,7 @@ class DataEditDialog(VeuszDialog):
         for a in self.datatableview.actions():
             a.setEnabled(model is not None)
 
-        self.datatableview.setModel(model)    
+        self.datatableview.setModel(model)
         self.setUnlinkState()
 
     def setUnlinkState(self):
@@ -711,8 +710,8 @@ class DataEditDialog(VeuszDialog):
         for name in self.dsbrowser.navtree.getSelectedDatasets():
             dataset = self.document.data[name]
             try:
-                recreate_register[type(dataset)](self.mainwindow, self.document,
-                                                 dataset, name)
+                recreate_register[type(dataset)](
+                    self.mainwindow, self.document, dataset, name)
             except KeyError:
                 pass
 
@@ -738,7 +737,7 @@ class DataEditDialog(VeuszDialog):
                     rowitems = []
                 lastrow = row
             rowitems.append(
-                cstr(model.createIndex(row, column).data()) )
+                str(model.createIndex(row, column).data()) )
         if rowitems:
             lines.append( '\t'.join(rowitems) )
         lines.append('')  # blank line at end
