@@ -21,11 +21,11 @@
 
 """A class to represent Veusz documents, with dataset classes."""
 
-from __future__ import division, print_function, absolute_import
 import codecs
 import os.path
 import traceback
 import datetime
+from io import StringIO
 from collections import defaultdict
 
 try:
@@ -33,7 +33,6 @@ try:
 except ImportError:
     h5py = None
 
-from ..compat import citems, cvalues, cstr, CStringIO, cexecfile
 from .. import qtall as qt
 
 from . import widgetfactory
@@ -59,7 +58,7 @@ def getSuitableParent(widgettype, initialwidget):
         parent = parent.parent
     return parent
 
-class DocSuspend(object):
+class DocSuspend:
     """Handle document updates/suspensions."""
     def __init__(self, doc):
         self.doc = doc
@@ -78,11 +77,16 @@ class Document(qt.QObject):
     # this is emitted when the document is modified
     signalModified = qt.pyqtSignal(int)
     # emited to log a message
-    sigLog = qt.pyqtSignal(cstr)
+    sigLog = qt.pyqtSignal(str)
     # emitted when document wiped
     sigWiped = qt.pyqtSignal()
-    # to ask whether the import is allowed (module name and symbol list)
-    sigAllowedImports = qt.pyqtSignal(cstr, list)
+    # to ask whether an import is allowed
+    sigAllowedImports = qt.pyqtSignal()
+    # security value set
+    sigSecuritySet = qt.pyqtSignal(bool)
+
+    # compatibility level
+    maxcompatlevel = 1
 
     def __init__(self):
         """Initialise the document."""
@@ -94,6 +98,7 @@ class Document(qt.QObject):
 
         # change tracking of document as a whole
         self.changeset = 0            # increased when the document changes
+        self.compatlevel = 0          # for non-backward compatible changes
 
         # map tags to dataset names
         self.datasettags = defaultdict(list)
@@ -114,8 +119,10 @@ class Document(qt.QObject):
     def wipe(self):
         """Wipe out any stored data."""
         self.data = {}
+
         self.basewidget = widgetfactory.thefactory.makeWidget(
             'document', None, self)
+
         self.setModified(False)
         self.filename = ""
         self.evaluate.wipe()
@@ -145,11 +152,25 @@ class Document(qt.QObject):
         """Return context manager for suspending updates."""
         return DocSuspend(self)
 
-    def makeDefaultDoc(self, mode='graph'):
+    def setCompatLevel(self, level):
+        """Update the compatiblity level.
+
+        If level<0, then use latest version
+        """
+        if level < 0:
+            level = self.maxcompatlevel
+        elif level > self.maxcompatlevel:
+            raise ValueError(
+                "Compatibility level greater than supported by this Veusz version")
+        self.compatlevel = level
+        self.basewidget.makeStylesheet(compatlevel=self.compatlevel)
+
+    def makeDefaultDoc(self, mode='graph', compatlevel=-1):
         """Add default widgets to create document.
 
         mode == 'graph', 'polar', 'ternary' or 'graph3d'
         """
+        self.setCompatLevel(compatlevel)
         page = widgetfactory.thefactory.makeWidget(
             'page', self.basewidget, self)
 
@@ -162,6 +183,7 @@ class Document(qt.QObject):
         self.setModified()
         self.setModified(False)
         self.changeset = 0
+        self.evaluate.setSecurity(True)
 
     def log(self, message):
         """Log a message - this is emitted as a signal."""
@@ -258,9 +280,10 @@ class Document(qt.QObject):
         if filenames is a set, only get the objects with filenames given
         """
         links = set()
-        for ds in cvalues(self.data):
-            if ds.linked and (filenames is None or
-                              ds.linked.filename in filenames):
+        for ds in self.data.values():
+            if ds.linked and (
+                    filenames is None or
+                    ds.linked.filename in filenames):
                 links.add(ds.linked)
         return list(links)
 
@@ -292,7 +315,7 @@ class Document(qt.QObject):
 
     def datasetName(self, dataset):
         """Find name for given dataset, raising ValueError if missing."""
-        for name, ds in citems(self.data):
+        for name, ds in self.data.items():
             if ds is dataset:
                 return name
         raise ValueError("Cannot find dataset")
@@ -334,7 +357,8 @@ class Document(qt.QObject):
 
         for plugin in pluginlist:
             try:
-                cexecfile(plugin, {})
+                with open(plugin) as f:
+                    exec(f.read())
             except Exception:
                 err = _('Error loading plugin %s\n\n%s') % (
                     plugin, traceback.format_exc())
@@ -362,7 +386,7 @@ class Document(qt.QObject):
     def datasetTags(self):
         """Get list of all tags in datasets."""
         tags = set()
-        for dataset in cvalues(self.data):
+        for dataset in self.data.values():
             tags.update(dataset.tags)
         return sorted(tags)
 
@@ -370,8 +394,9 @@ class Document(qt.QObject):
         """Write a header to a saved file of type."""
 
         fileobj.write('# Veusz %s (version %s)\n' % (type, utils.version()))
-        fileobj.write('# Saved at %s\n\n' %
-                      datetime.datetime.utcnow().isoformat())
+        fileobj.write(
+            '# Saved at %s\n\n' %
+            datetime.datetime.utcnow().isoformat())
 
     def saveDatasetTags(self, fileobj):
         """Write dataset tags to output file"""
@@ -401,11 +426,15 @@ class Document(qt.QObject):
 
         self._writeFileHeader(fileobj, 'saved document')
 
+        # write compatibility level
+        fileobj.write('SetCompatLevel(%s)\n' % self.compatlevel)
+
         # add file directory to import path if we know it
         reldirname = None
         if getattr(fileobj, 'name', False):
             reldirname = os.path.dirname( os.path.abspath(fileobj.name) )
-            fileobj.write('AddImportPath(%s)\n' % utils.rrepr(reldirname))
+            if setting.settingdb['docfile_addimportpaths']:
+                fileobj.write('AddImportPath(%s)\n' % utils.rrepr(reldirname))
 
         # save those datasets which are linked
         # we do this first in case the datasets are overridden below
@@ -440,7 +469,7 @@ class Document(qt.QObject):
         datagrp = vszgrp.create_group('Data')
         docgrp = vszgrp.create_group('Document')
 
-        textstream = CStringIO()
+        textstream = StringIO()
 
         self._writeFileHeader(textstream, 'saved document')
 
@@ -457,8 +486,8 @@ class Document(qt.QObject):
         # we do this first in case the datasets are overridden below
         savedlinks = {}
         for name, dataset in sorted(self.data.items()):
-            dataset.saveLinksToSavedDoc(textstream, savedlinks,
-                                        relpath=reldirname)
+            dataset.saveLinksToSavedDoc(
+                textstream, savedlinks, relpath=reldirname)
 
         # save the remaining datasets
         for name, dataset in sorted(self.data.items()):
@@ -637,16 +666,19 @@ class Document(qt.QObject):
 
             # do the widget's children
             for w in root.children:
-                self.walkNodes(tocall, root=w, nodetypes=nodetypes,
-                               _path = _path + '/' + w.name)
+                self.walkNodes(
+                    tocall, root=w, nodetypes=nodetypes,
+                    _path=_path+'/'+w.name)
             # then do the widget's settings
-            self.walkNodes(tocall, root=root.settings,
-                           nodetypes=nodetypes, _path=_path)
+            self.walkNodes(
+                tocall, root=root.settings,
+                nodetypes=nodetypes, _path=_path)
         elif root.nodetype == 'settings':
             # do the settings of the settings
             for name, s in sorted(root.setdict.items()):
-                self.walkNodes(tocall, root=s, nodetypes=nodetypes,
-                               _path = _path + '/' + s.name)
+                self.walkNodes(
+                    tocall, root=s, nodetypes=nodetypes,
+                    _path=_path+'/'+s.name)
         # elif root.nodetype == 'setting': pass
 
     def formatValsWithDatatypeToText(self, vals, displaydatatype):
@@ -656,8 +688,9 @@ class Document(qt.QObject):
         if displaydatatype == 'text':
             return vals
         elif displaydatatype == 'numeric':
-            return [ utils.formatNumber(val, '%Vg', locale=self.locale)
-                     for val in vals ]
+            return [
+                utils.formatNumber(val, '%Vg', locale=self.locale)
+                for val in vals ]
         elif displaydatatype == 'date':
             return [ utils.dateFloatToString(val) for val in vals ]
         else:
