@@ -18,13 +18,11 @@
 
 '''An axis which can be broken in places.'''
 
-from __future__ import division
 import bisect
 
 import numpy as N
 
-from ..compat import crange, czip
-from .. import qtall as qt4
+from .. import qtall as qt
 from .. import setting
 from .. import document
 from .. import utils
@@ -34,7 +32,7 @@ from . import controlgraph
 
 def _(text, disambiguation=None, context='BrokenAxis'):
     '''Translate text.'''
-    return qt4.QCoreApplication.translate(context, text, disambiguation)
+    return qt.QCoreApplication.translate(context, text, disambiguation)
 
 class AxisBroken(axis.Axis):
     '''An axis widget which can have gaps in it.'''
@@ -48,27 +46,22 @@ class AxisBroken(axis.Axis):
         self.rangeswitch = None
         self.breakchangeset = -1
 
-        if type(self) == AxisBroken:
-            self.readDefaults()
-
     @classmethod
     def addSettings(klass, s):
         '''Construct list of settings.'''
         axis.Axis.addSettings(s)
 
         s.add( setting.FloatList(
-                'breakPoints',
-                [],
-                descr = _('Pairs of values to start and stop breaks'),
-                usertext = _('Break pairs'),
-                ), 4 )
+            'breakPoints',
+            [],
+            descr = _('Pairs of values to start and stop breaks'),
+            usertext = _('Break pairs')), 4 )
         s.add( setting.FloatList(
-                'breakPosns',
-                [],
-                descr = _('Positions (fractions) along axis where to break'),
-                usertext = _('Break positions'),
-                formatting=True,
-                ) )
+            'breakPosns',
+            [],
+            descr = _('Positions (fractions) along axis where to break'),
+            usertext = _('Break positions'),
+            formatting=True) )
 
     def switchBreak(self, num, posn, otherposition=None):
         """Switch to break given (or None to disable)."""
@@ -107,7 +100,7 @@ class AxisBroken(axis.Axis):
 
         # first work out which break region the values are in
         out = []
-        for sval, val in czip(svals, vals):
+        for sval, val in zip(svals, vals):
             # find index for appropriated scaled starting value
             breaki = bisect.bisect_left(self.posstarts, sval) - 1
 
@@ -203,9 +196,10 @@ class AxisBroken(axis.Axis):
             lowerupper = ( self.posstarts[self.rangeswitch],
                            self.posstops[self.rangeswitch] )
 
-        axis.Axis.updateAxisLocation(self, bounds,
-                                     otherposition=otherposition,
-                                     lowerupperposition=lowerupper)
+        axis.Axis.updateAxisLocation(
+            self, bounds,
+            otherposition=otherposition,
+            lowerupperposition=lowerupper)
 
     def computePlottedRange(self):
         """Given range of data, recompute stops and start values of
@@ -221,7 +215,7 @@ class AxisBroken(axis.Axis):
 
         # filter to range
         newpoints = []
-        for i in crange(0, len(points)//2 * 2, 2):
+        for i in range(0, len(points)//2 * 2, 2):
             if points[i] >= min(r) and points[i+1] <= max(r):
                 newpoints += [points[i], points[i+1]]
 
@@ -230,13 +224,13 @@ class AxisBroken(axis.Axis):
             self.plottedrange[1]]
 
         # axis values for starting and stopping
-        self.breakvstarts = [ self.breakvlist[i*2] for i in crange(num) ]
-        self.breakvstops = [ self.breakvlist[i*2+1] for i in crange(num) ]
+        self.breakvstarts = [ self.breakvlist[i*2] for i in range(num) ]
+        self.breakvstops = [ self.breakvlist[i*2+1] for i in range(num) ]
 
         # compute ticks for each range
         self.minorticklist = []
         self.majorticklist = []
-        for i in crange(self.breakvnum):
+        for i in range(self.breakvnum):
             self.plottedrange = [self.breakvstarts[i], self.breakvstops[i]]
             reverse = self.plottedrange[0] > self.plottedrange[1]
             if reverse:
@@ -263,9 +257,9 @@ class AxisBroken(axis.Axis):
         # temporarily change position of axis to other side for drawing
         self.updateAxisLocation(posn, otherposition=otheredge)
         if not s.Line.hide:
-            self._drawAxisLine(painter)
+            self._drawAxisLine(painter, posn)
 
-        for i in crange(self.breakvnum):
+        for i in range(self.breakvnum):
             self.switchBreak(i, posn, otherposition=otheredge)
 
             # plot coordinates of ticks
@@ -279,7 +273,7 @@ class AxisBroken(axis.Axis):
 
         self.switchBreak(None, posn)
 
-    def _drawAxisLine(self, painter):
+    def _drawAxisLine(self, painter, posn):
         """Draw the line of the axis, indicating broken positions.
         We currently use a triangle to mark the broken position
         """
@@ -294,7 +288,7 @@ class AxisBroken(axis.Axis):
             markdirn = -markdirn
 
         # add shape for each break
-        for start, stop in czip( self.posstarts[1:], self.posstops[:-1] ):
+        for start, stop in zip( self.posstarts[1:], self.posstops[:-1] ):
             p1 += [stop, (start+stop)*0.5, start]
             p2 += [0, markdirn*(start-stop)*0.5, 0]
 
@@ -303,19 +297,27 @@ class AxisBroken(axis.Axis):
         p2.append(0.)
 
         # scale points by length of axis and add correct origin
-        scale = self.coordParr2 - self.coordParr1
-        p1 = N.array(p1) * scale + self.coordParr1
-        p2 = N.array(p2) * scale + self.coordPerp
+        s = self.settings
 
-        if self.settings.direction == 'vertical':
+        if s.direction == 'vertical':
+            delta = posn[1]-posn[3]
+            minv = posn[3]
+        else:
+            delta = posn[2]-posn[0]
+            minv = posn[0]
+
+        p1 = N.array(p1)*delta + minv
+        p2 = N.array(p2)*delta + self.coordPerp
+
+        if s.direction == 'vertical':
             p1, p2 = p2, p1
 
         # convert to polygon and draw
-        poly = qt4.QPolygonF()
+        poly = qt.QPolygonF()
         utils.addNumpyToPolygonF(poly, p1, p2)
 
-        pen = self.settings.get('Line').makeQPen(painter)
-        pen.setCapStyle(qt4.Qt.FlatCap)
+        pen = s.get('Line').makeQPen(painter)
+        pen.setCapStyle(qt.Qt.FlatCap)
         painter.setPen(pen)
         painter.drawPolyline(poly)
 
@@ -339,20 +341,22 @@ class AxisBroken(axis.Axis):
 
         with painter:
             painter.save()
-            painter.setClipRect( qt4.QRectF(
-                    qt4.QPointF(parentposn[0], parentposn[1]),
-                    qt4.QPointF(parentposn[2], parentposn[3]) ) )
+            painter.setClipRect( qt.QRectF(
+                qt.QPointF(parentposn[0], parentposn[1]),
+                qt.QPointF(parentposn[2], parentposn[3]) ) )
 
-            for i in crange(self.breakvnum):
+            for i in range(self.breakvnum):
                 self.switchBreak(i, parentposn)
                 if not s.MinorGridLines.hide:
                     coordminorticks = self._graphToPlotter(self.minorticklist[i])
-                    self._drawGridLines('MinorGridLines', painter, coordminorticks,
-                                        parentposn)
+                    self._drawGridLines(
+                        'MinorGridLines', painter, coordminorticks,
+                        parentposn)
                 if not s.GridLines.hide:
                     coordticks = self._graphToPlotter(self.majorticklist[i])
-                    self._drawGridLines('GridLines', painter, coordticks,
-                                        parentposn)
+                    self._drawGridLines(
+                        'GridLines', painter, coordticks,
+                        parentposn)
 
             self.switchBreak(None, parentposn)
             painter.restore()
@@ -375,12 +379,16 @@ class AxisBroken(axis.Axis):
 
         # plot the line along the axis
         if not s.Line.hide:
-            self._drawAxisLine(painter)
+            self._drawAxisLine(painter, posn)
 
         max_delta = 0
-        for i in crange(self.breakvnum):
+        suppresstext = False
+        for i in range(self.breakvnum):
             self.switchBreak(i, posn)
 
+            if self.plottedrange[0]==self.plottedrange[1]:
+                continue
+
             # plot coordinates of ticks
             coordticks = self._graphToPlotter(self.majorticklist[i])
             coordminorticks = self._graphToPlotter(self.minorticklist[i])
@@ -398,9 +406,10 @@ class AxisBroken(axis.Axis):
             # plot tick labels
             suppresstext = self._suppressText(painter, parentposn, outerbounds)
             if not s.TickLabels.hide and not suppresstext:
-                self._drawTickLabels(phelper, painter, coordticks, sign,
-                                     outerbounds, self.majorticklist[i],
-                                     texttorender)
+                self._drawTickLabels(
+                    phelper, painter, coordticks, sign,
+                    outerbounds, self.majorticklist[i],
+                    texttorender)
 
             # this is the maximum delta of any of the breaks
             max_delta = max(max_delta, self._delta_axis)
@@ -416,8 +425,8 @@ class AxisBroken(axis.Axis):
 
         # make control item for axis
         phelper.setControlGraph(self, [ controlgraph.ControlAxisLine(
-                    self, self.settings.direction, self.coordParr1,
-                    self.coordParr2, self.coordPerp, posn) ])
+            self, phelper, self.settings.direction, self.coordParr1,
+            self.coordParr2, self.coordPerp, posn) ])
 
 # allow the factory to instantiate the widget
-document.thefactory.register( AxisBroken )
+document.thefactory.register(AxisBroken)
