@@ -22,12 +22,10 @@ Contour plotting requires that the veusz_helpers package is installed,
 as a C routine (taken from matplotlib) is used to trace the contours.
 """
 
-from __future__ import division, print_function
 import sys
 import math
 
-from ..compat import czip, crange
-from .. import qtall as qt4
+from .. import qtall as qt
 import numpy as N
 
 from .. import setting
@@ -36,16 +34,12 @@ from .. import utils
 
 from . import plotters
 
-try:
-    from ..helpers._nc_cntr import Cntr
-    from ..helpers.qtloops import LineLabeller
-except ImportError:
-    Cntr = None
-    LineLabeller = object   # allow class definition below
+from ..helpers._nc_cntr import Cntr
+from ..helpers.qtloops import LineLabeller
 
 def _(text, disambiguation=None, context='Contour'):
     """Translate text."""
-    return qt4.QCoreApplication.translate(context, text, disambiguation)
+    return qt.QCoreApplication.translate(context, text, disambiguation)
 
 def finitePoly(poly):
     """Remove non-finite coordinates from numpy arrays of coordinates."""
@@ -57,13 +51,14 @@ def finitePoly(poly):
     return out
 
 class ContourLineLabeller(LineLabeller):
-    def __init__(self, clip, rot, painter, font):
+    def __init__(self, clip, rot, painter, font, doc):
         LineLabeller.__init__(self, clip, rot)
-        self.clippath = qt4.QPainterPath()
+        self.clippath = qt.QPainterPath()
         self.clippath.addRect(clip)
         self.labels = []
         self.painter = painter
         self.font = font
+        self.document = doc
 
     def drawAt(self, idx, rect):
         """Called to draw the label with the index given."""
@@ -79,10 +74,12 @@ class ContourLineLabeller(LineLabeller):
             self.painter, self.font,
             rect.cx, rect.cy, text,
             alignhorz=0, alignvert=0,
-            angle=angle)
+            angle=angle,
+            doc=self.document)
+
         rend.render()
         if rect.xw > 0:
-            p = qt4.QPainterPath()
+            p = qt.QPainterPath()
             p.addPolygon(rect.makePolygon())
             self.clippath -= p
 
@@ -91,65 +88,72 @@ class ContourFills(setting.Settings):
     def __init__(self, name, **args):
         setting.Settings.__init__(self, name, **args)
         self.add( setting.FillSet(
-                'fills', [],
-                descr = _('Fill styles to plot between contours'),
-                usertext=_('Fill styles'),
-                formatting=True) )
-        self.add( setting.Bool('hide', False,
-                               descr = _('Hide fills'),
-                               usertext = _('Hide'),
-                               formatting = True) )
-        
+            'fills', [],
+            descr=_('Fill styles to plot between contours'),
+            usertext=_('Fill styles'),
+            formatting=True) )
+        self.add( setting.Bool(
+            'hide', False,
+            descr=_('Hide fills'),
+            usertext=_('Hide'),
+            formatting=True) )
+
 class ContourLines(setting.Settings):
     """Settings for contour lines."""
     def __init__(self, name, **args):
         setting.Settings.__init__(self, name, **args)
         self.add( setting.LineSet(
-                'lines',
-                [('solid', '1pt', 'black', False)],
-                descr = _('Line styles to plot the contours '
-                          'using'), usertext=_('Line styles'),
-                formatting=True) )
-        self.add( setting.Bool('hide', False,
-                               descr = _('Hide lines'),
-                               usertext = _('Hide'),
-                               formatting = True) )
+            'lines',
+            [('solid', '1pt', 'black', False)],
+            descr=_('Line styles to plot the contours using'),
+            usertext=_('Line styles'),
+            formatting=True) )
+        self.add( setting.Bool(
+            'hide', False,
+            descr=_('Hide lines'),
+            usertext=_('Hide'),
+            formatting=True) )
 
 class SubContourLines(setting.Settings):
     """Sub-dividing contour line settings."""
     def __init__(self, name, **args):
         setting.Settings.__init__(self, name, **args)
         self.add( setting.LineSet(
-                'lines',
-                [('dot1', '1pt', 'black', False)],
-                descr = _('Line styles used for sub-contours'),
-                usertext=_('Line styles'),
-                formatting=True) )
-        self.add( setting.Int('numLevels', 5,
-                              minval=2,
-                              descr=_('Number of sub-levels to plot between '
-                                      'each contour'),
-                              usertext='Levels') )
-        self.add( setting.Bool('hide', True,
-                               descr=_('Hide lines'),
-                               usertext=_('Hide'),
-                               formatting=True) )
+            'lines',
+            [('dot1', '1pt', 'black', False)],
+            descr=_('Line styles used for sub-contours'),
+            usertext=_('Line styles'),
+            formatting=True) )
+        self.add( setting.Int(
+            'numLevels', 5,
+            minval=2,
+            descr=_('Number of sub-levels to plot between each contour'),
+            usertext='Levels') )
+        self.add( setting.Bool(
+            'hide', True,
+            descr=_('Hide lines'),
+            usertext=_('Hide'),
+            formatting=True) )
 
 class ContourLabel(setting.Text):
     """For tick labels on axes."""
 
     def __init__(self, name, **args):
         setting.Text.__init__(self, name, **args)
-        self.add( setting.Str( 'format', '%.3Vg',
-                               descr = _('Format of the tick labels'),
-                               usertext=_('Format')) )
-        self.add( setting.Float('scale', 1.,
-                                descr=_('A scale factor to apply to the values '
-                                        'of the tick labels'),
-                                usertext=_('Scale')) )
-        self.add( setting.Bool('rotate', True,
-                               descr=_('Rotate labels to follow lines'),
-                               usertext=_('Rotate')) )
+        self.add( setting.Str(
+            'format', '%.3Vg',
+            descr=_('Format of the tick labels'),
+            usertext=_('Format')) )
+        self.add( setting.Float(
+            'scale', 1.,
+            descr=_(
+                'A scale factor to apply to the values of the tick labels'),
+            usertext=_('Scale')) )
+        self.add( setting.Bool(
+            'rotate',
+            True,
+            descr=_('Rotate labels to follow lines'),
+            usertext=_('Rotate')) )
 
         self.get('hide').newDefault(True)
 
@@ -166,13 +170,7 @@ class Contour(plotters.GenericPlotter):
 
         plotters.GenericPlotter.__init__(self, parent, name=name)
 
-        if Cntr is None:
-            print(('WARNING: Veusz cannot import contour module\n'
-                                'Please run python setup.py build\n'
-                                'Contour support is disabled'), file=sys.stderr)
-            
         # keep track of settings so we recalculate when necessary
-        self.lastdataset = None
         self.contsettings = None
 
         # cached traced contours
@@ -180,73 +178,76 @@ class Contour(plotters.GenericPlotter):
         self._cachedpolygons = None
         self._cachedsubcontours = None
 
-        if type(self) == Contour:
-            self.readDefaults()
-
     @classmethod
     def addSettings(klass, s):
         """Construct list of settings."""
         plotters.GenericPlotter.addSettings(s)
 
-        s.add( setting.Dataset('data', '',
-                               dimensions = 2,
-                               descr = _('Dataset to plot'),
-                               usertext=_('Dataset')),
-               0 )
-        s.add( setting.FloatOrAuto('min', 'Auto',
-                                   descr = _('Minimum value of contour scale'),
-                                   usertext=_('Min. value')),
-               1 )
-        s.add( setting.FloatOrAuto('max', 'Auto',
-                                   descr = _('Maximum value of contour scale'),
-                                   usertext=_('Max. value')),
-               2 )
-        s.add( setting.Int('numLevels', 5,
-                           minval = 1,
-                           descr = _('Number of contour levels to plot'),
-                           usertext=_('Number levels')),
-               3 )
-        s.add( setting.Choice('scaling',
-                              ['linear', 'sqrt', 'log', 'squared', 'manual'],
-                              'linear',
-                              descr = _('Scaling between contour levels'),
-                              usertext=_('Scaling')),
-               4 )
-        s.add( setting.FloatList('manualLevels',
-                                 [],
-                                 descr = _('Levels to use for manual scaling'),
-                                 usertext=_('Manual levels')),
-               5 )
+        s.add( setting.DatasetExtended(
+            'data', '',
+            dimensions=2,
+            descr=_('Dataset to plot'),
+            usertext=_('Dataset')), 0 )
+        s.add( setting.FloatOrAuto(
+            'min', 'Auto',
+            descr=_('Minimum value of contour scale'),
+            usertext=_('Min. value')), 1 )
+        s.add( setting.FloatOrAuto(
+            'max', 'Auto',
+            descr=_('Maximum value of contour scale'),
+            usertext=_('Max. value')), 2 )
+        s.add( setting.Int(
+            'numLevels', 5,
+            minval=1,
+            descr=_('Number of contour levels to plot'),
+            usertext=_('Number levels')), 3 )
+        s.add( setting.Choice(
+            'scaling',
+            ['linear', 'sqrt', 'log', 'squared', 'manual'],
+            'linear',
+            descr=_('Scaling between contour levels'),
+            usertext=_('Scaling')), 4 )
+        s.add( setting.FloatList(
+            'manualLevels',
+            [],
+            descr=_('Levels to use for manual scaling'),
+            usertext=_('Manual levels')), 5 )
 
-        s.add( setting.Bool('keyLevels', False, descr=_('Show levels in key'),
-                            usertext=_('Levels in key')),
-               6 )
+        s.add( setting.Bool(
+            'keyLevels',
+            False,
+            descr=_('Show levels in key'),
+            usertext=_('Levels in key')), 6 )
 
-        s.add( setting.FloatList('levelsOut',
-                                 [],
-                                 descr = _('Levels used in the plot'),
-                                 usertext=_('Output levels')),
-               7, readonly=True )
+        s.add( setting.FloatList(
+            'levelsOut',
+            [],
+            descr=_('Levels used in the plot'),
+            usertext=_('Output levels')), 7, readonly=True )
 
-        s.add( ContourLabel('ContourLabels',
-                            descr = _('Contour label settings'),
-                            usertext = _('Contour labels')),
-               pixmap = 'settings_axisticklabels' )
+        s.add( ContourLabel(
+            'ContourLabels',
+            descr=_('Contour label settings'),
+            usertext=_('Contour labels')),
+            pixmap='settings_axisticklabels' )
 
-        s.add( ContourLines('Lines',
-                            descr=_('Contour lines'),
-                            usertext=_('Contour lines')),
-               pixmap = 'settings_contourline' )
+        s.add( ContourLines(
+            'Lines',
+            descr=_('Contour lines'),
+            usertext=_('Contour lines')),
+            pixmap='settings_contourline' )
 
-        s.add( ContourFills('Fills',
-                            descr=_('Fill within contours'),
-                            usertext=_('Contour fills')),
-               pixmap = 'settings_contourfill' )
+        s.add( ContourFills(
+            'Fills',
+            descr=_('Fill within contours'),
+            usertext=_('Contour fills')),
+            pixmap='settings_contourfill' )
 
-        s.add( SubContourLines('SubLines',
-                               descr=_('Sub-contour lines'),
-                               usertext=_('Sub-contour lines')),
-               pixmap = 'settings_subcontourline' )
+        s.add( SubContourLines(
+            'SubLines',
+            descr=_('Sub-contour lines'),
+            usertext=_('Sub-contour lines')),
+            pixmap='settings_subcontourline' )
 
         s.add( setting.SettingBackwardCompat('lines', 'Lines/lines', None) )
         s.add( setting.SettingBackwardCompat('fills', 'Fills/fills', None) )
@@ -262,7 +263,7 @@ class Contour(plotters.GenericPlotter):
             out.append( s.data )
         if s.scaling == 'manual':
             out.append('manual levels (%s)' %  (
-                    ', '.join([str(i) for i in s.manualLevels])))
+                ', '.join([str(i) for i in s.manualLevels])))
         else:
             out.append('%(numLevels)i %(scaling)s levels (%(min)s to %(max)s)' % s)
         return ', '.join(out)
@@ -278,14 +279,16 @@ class Contour(plotters.GenericPlotter):
         d = self.document
 
         minval, maxval = 0., 1.
-        if s.data in d.data:
-            # scan data
-            data = d.data[s.data].data
-            minval, maxval = N.nanmin(data), N.nanmax(data)
-            if not N.isfinite(minval):
-                minval = 0.
-            if not N.isfinite(maxval):
-                maxval = 1.
+        # scan data
+        data = s.get('data').getData(d)
+        if data is None or data.dimensions != 2 or data.data.size == 0:
+            return
+
+        minval, maxval = N.nanmin(data.data), N.nanmax(data.data)
+        if not N.isfinite(minval):
+            minval = 0.
+        if not N.isfinite(maxval):
+            maxval = 1.
 
         # override if not auto
         if s.min != 'Auto':
@@ -304,7 +307,7 @@ class Contour(plotters.GenericPlotter):
             if minval == maxval:
                 minval = 0.
                 maxval = 1.
-                
+
             # calculate levels for each scaling
             if scaling == 'linear':
                 delta = (maxval - minval) / (numlevels-1)
@@ -360,7 +363,7 @@ class Contour(plotters.GenericPlotter):
         elif scaling == 'manual':
             drange = N.arange(1, num)
             out = [[]]
-            for conmin, conmax in czip(levels[:-1], levels[1:]):
+            for conmin, conmax in zip(levels[:-1], levels[1:]):
                 delta = (conmax-conmin) / num
                 out.append( conmin+drange*delta )
             slev = N.hstack(out)
@@ -380,8 +383,8 @@ class Contour(plotters.GenericPlotter):
         d = self.document
 
         # return if no data or if the dataset isn't two dimensional
-        data = d.data.get(s.data, None)
-        if data is None or data.dimensions != 2:
+        data = s.get('data').getData(d)
+        if data is None or data.dimensions != 2 or data.data.size == 0:
             return
 
         xr, yr = data.getDataRanges()
@@ -405,9 +408,10 @@ class Contour(plotters.GenericPlotter):
         s = self.settings
         if s.keyLevels:
             cl = s.get('ContourLabels')
-            return utils.formatNumber( s.levelsOut[number] * cl.scale,
-                                       cl.format,
-                                       locale=self.document.locale )
+            return utils.formatNumber(
+                s.levelsOut[number] * cl.scale,
+                cl.format,
+                locale=self.document.locale )
         else:
             return ''
 
@@ -426,21 +430,24 @@ class Contour(plotters.GenericPlotter):
         d = self.document
 
         # return if no data or if the dataset isn't two dimensional
-        data = d.data.get(s.data, None)
+        data = s.get('data').getData(d)
         if data is None or data.dimensions != 2 or data.data.size == 0:
-            self.contsettings = self.lastdataset = None
+            self.contsettings = None
             s.levelsOut = []
             return False
 
-        contsettings = ( s.min, s.max, s.numLevels, s.scaling,
-                         s.SubLines.numLevels,
-                         len(s.Fills.fills) == 0 or s.Fills.hide,
-                         len(s.SubLines.lines) == 0 or s.SubLines.hide,
-                         tuple(s.manualLevels) )
+        hashval = hash(bytes(data.data))
+        contsettings = (
+            s.min, s.max, s.numLevels, s.scaling,
+            s.SubLines.numLevels,
+            len(s.Fills.fills) == 0 or s.Fills.hide,
+            len(s.SubLines.lines) == 0 or s.SubLines.hide,
+            tuple(s.manualLevels),
+            hashval
+        )
 
-        if data is not self.lastdataset or contsettings != self.contsettings:
+        if contsettings != self.contsettings:
             self.updateContours()
-            self.lastdataset = data
             self.contsettings = contsettings
 
         return True
@@ -466,13 +473,12 @@ class Contour(plotters.GenericPlotter):
         sublevels = self.calculateSubLevels(minval, maxval, levels)
 
         # find coordinates of image coordinate bounds
-        data = d.data[s.data]
-        rangex, rangey = data.getDataRanges()
-        yw, xw = data.data.shape
-
-        if xw == 0 or yw == 0:
+        data = s.get('data').getData(d)
+        if data is None or data.dimensions != 2 or data.data.size == 0:
             return
 
+        rangex, rangey = data.getDataRanges()
+        yw, xw = data.data.shape
         xc, yc = data.getPixelCentres()
         xpts = N.reshape( N.tile(xc, yw), (yw, xw) )
         ypts = N.tile(yc[:, N.newaxis], xw)
@@ -498,7 +504,7 @@ class Contour(plotters.GenericPlotter):
             # trace the polygons between the contours
             if len(s.Fills.fills) != 0 and len(levels) > 1 and not s.Fills.hide:
                 self._cachedpolygons = []
-                for level1, level2 in czip(levels[:-1], levels[1:]):
+                for level1, level2 in zip(levels[:-1], levels[1:]):
                     linelist = c.trace(level1, level2)
                     self._cachedpolygons.append( finitePoly(linelist) )
 
@@ -522,34 +528,37 @@ class Contour(plotters.GenericPlotter):
 
         cl = s.get('ContourLabels')
         font = cl.makeQFont(painter)
-        labelpen = cl.makeQPen()
-        descent = qt4.QFontMetricsF(font).descent()
+        labelpen = cl.makeQPen(painter)
+        descent = qt.QFontMetricsF(font).descent()
 
         # linelabeller does clipping and labelling of contours
-        linelabeller = ContourLineLabeller(clip, cl.rotate, painter, font)
+        linelabeller = ContourLineLabeller(
+            clip, cl.rotate, painter, font, self.document)
         levels = []
 
         # iterate over each level, and list of lines
         for num, linelist in enumerate(contours):
 
-            if showlabels:
+            if showlabels and num<len(s.levelsOut):
                 number = s.levelsOut[num]
-                text = utils.formatNumber(number * cl.scale, cl.format,
-                                          locale=self.document.locale)
-                rend = utils.Renderer(painter, font, 0, 0, text, alignhorz=0,
-                                      alignvert=0, angle=0)
-                textdims = qt4.QSizeF(*rend.getDimensions())
-                textdims += qt4.QSizeF(descent*2, descent*2)
+                text = utils.formatNumber(
+                    number * cl.scale, cl.format,
+                    locale=self.document.locale)
+                rend = utils.Renderer(
+                    painter, font, 0, 0, text, alignhorz=0,
+                    alignvert=0, angle=0, doc=self.document)
+                textdims = qt.QSizeF(*rend.getDimensions())
+                textdims += qt.QSizeF(descent*2, descent*2)
             else:
-                textdims = qt4.QSizeF(0, 0)
+                textdims = qt.QSizeF(0, 0)
 
             # iterate over each complete line of the contour
             for curve in linelist:
                 # convert coordinates from graph to plotter
                 xplt = axes[0].dataToPlotterCoords(posn, curve[:,0])
                 yplt = axes[1].dataToPlotterCoords(posn, curve[:,1])
-                    
-                pts = qt4.QPolygonF()
+
+                pts = qt.QPolygonF()
                 utils.addNumpyToPolygonF(pts, xplt, yplt)
                 linelabeller.addLine(pts, textdims)
 
@@ -564,7 +573,7 @@ class Contour(plotters.GenericPlotter):
         linelabeller.process()
         painter.setClipPath(linelabeller.clippath)
 
-        for i in crange(linelabeller.getNumPolySets()):
+        for i in range(linelabeller.getNumPolySets()):
             polyset = linelabeller.getPolySet(i)
             painter.setPen(linestyles.makePen(painter, levels[i]))
             for poly in polyset:
@@ -575,16 +584,20 @@ class Contour(plotters.GenericPlotter):
     def plotContours(self, painter, posn, axes, clip):
         """Plot the traced contours on the painter."""
         s = self.settings
-        self._plotContours(painter, posn, axes, s.Lines.get('lines'),
-                           self._cachedcontours,
-                           not s.ContourLabels.hide, s.Lines.hide, clip)
+        self._plotContours(
+            painter, posn, axes, s.Lines.get('lines'),
+            self._cachedcontours,
+            not s.ContourLabels.hide, s.Lines.hide, clip
+        )
 
     def plotSubContours(self, painter, posn, axes, clip):
         """Plot sub contours on painter."""
         s = self.settings
-        self._plotContours(painter, posn, axes, s.SubLines.get('lines'),
-                           self._cachedsubcontours,
-                           False, s.SubLines.hide, clip)
+        self._plotContours(
+            painter, posn, axes, s.SubLines.get('lines'),
+            self._cachedsubcontours,
+            False, s.SubLines.hide, clip
+        )
 
     def plotContourFills(self, painter, posn, axes, clip):
         """Plot the traced contours on the painter."""
@@ -599,16 +612,16 @@ class Contour(plotters.GenericPlotter):
         for num, polylist in enumerate(self._cachedpolygons):
 
             # iterate over each complete line of the contour
-            path = qt4.QPainterPath()
+            path = qt.QPainterPath()
             for poly in polylist:
                 # convert coordinates from graph to plotter
                 xplt = axes[0].dataToPlotterCoords(posn, poly[:,0])
                 yplt = axes[1].dataToPlotterCoords(posn, poly[:,1])
 
-                pts = qt4.QPolygonF()
+                pts = qt.QPolygonF()
                 utils.addNumpyToPolygonF(pts, xplt, yplt)
 
-                clippedpoly = qt4.QPolygonF()
+                clippedpoly = qt.QPolygonF()
                 utils.polygonClip(pts, clip, clippedpoly)
                 path.addPolygon(clippedpoly)
 
@@ -617,4 +630,4 @@ class Contour(plotters.GenericPlotter):
             utils.brushExtFillPath(painter, brush, path)
 
 # allow the factory to instantiate a contour
-document.thefactory.register( Contour )
+document.thefactory.register(Contour)
