@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #    Copyright (C) 2003 Jeremy S. Sanders
 #    Email: Jeremy Sanders <jeremy@jeremysanders.net>
 #
@@ -19,11 +18,9 @@
 
 """Implements the main window of the application."""
 
-from __future__ import division, print_function
 import os
 import os.path
 import sys
-import glob
 import re
 import datetime
 
@@ -32,7 +29,6 @@ try:
 except ImportError:
     h5py = None
 
-from ..compat import cstr, cstrerror, cgetcwd, cbytes
 from .. import qtall as qt
 
 from .. import document
@@ -40,6 +36,7 @@ from .. import utils
 from ..utils import vzdbus
 from .. import setting
 from .. import plugins
+from ..qtwidgets.clicklabel import ClickLabel
 
 from . import consolewindow
 from . import plotwindow
@@ -131,7 +128,7 @@ class MainWindow(qt.QMainWindow):
         # icon and different size variations
         self.setWindowIcon( utils.getIcon('veusz') )
 
-        # master documenent
+        # master document
         self.document = document.Document()
 
         # filename for document and update titlebar
@@ -145,26 +142,26 @@ class MainWindow(qt.QMainWindow):
         self._defineMenus()
 
         # make plot window
-        self.plot = plotwindow.PlotWindow(self.document, self,
-                                          menu = self.menus['view'])
+        self.plot = plotwindow.PlotWindow(
+            self.document, self, menu=self.menus['view'])
         self.setCentralWidget(self.plot)
         self.plot.showToolbar()
 
         # likewise with the tree-editing window
         self.treeedit = treeeditwindow.TreeEditDock(self.document, self)
         self.addDockWidget(qt.Qt.LeftDockWidgetArea, self.treeedit)
-        self.propdock = treeeditwindow.PropertiesDock(self.document,
-                                                      self.treeedit, self)
+        self.propdock = treeeditwindow.PropertiesDock(
+            self.document, self.treeedit, self)
         self.addDockWidget(qt.Qt.LeftDockWidgetArea, self.propdock)
-        self.formatdock = treeeditwindow.FormatDock(self.document,
-                                                    self.treeedit, self)
+        self.formatdock = treeeditwindow.FormatDock(
+            self.document, self.treeedit, self)
         self.addDockWidget(qt.Qt.LeftDockWidgetArea, self.formatdock)
         self.datadock = DataNavigatorWindow(self.document, self, self)
         self.addDockWidget(qt.Qt.RightDockWidgetArea, self.datadock)
 
         # make the console window a dock
-        self.console = consolewindow.ConsoleWindow(self.document,
-                                                   self)
+        self.console = consolewindow.ConsoleWindow(
+            self.document, self)
         self.console.hide()
         self.interpreter = self.console.interpreter
         self.addDockWidget(qt.Qt.BottomDockWidgetArea, self.console)
@@ -199,10 +196,17 @@ class MainWindow(qt.QMainWindow):
         statusbar.addPermanentWidget(self.pagelabel)
         self.pagelabel.show()
 
+        # security label
+        self.securitylabel = ClickLabel(_("Untrusted mode"), statusbar)
+        statusbar.addPermanentWidget(self.securitylabel)
+        self.securitylabel.show()
+        self.document.sigSecuritySet.connect(self.slotUpdateSecurity)
+        self.securitylabel.clicked.connect(self.slotFileTrust)
+
         # working directory - use previous one
         self.dirname = setdb.get('dirname', qt.QDir.homePath())
         if setdb['dirname_usecwd']:
-            self.dirname = cgetcwd()
+            self.dirname = os.getcwd()
 
         # connect plot signals to main window
         self.plot.sigUpdatePage.connect(self.slotUpdatePage)
@@ -222,7 +226,7 @@ class MainWindow(qt.QMainWindow):
         # enable/disable undo/redo
         self.menus['edit'].aboutToShow.connect(self.slotAboutToShowEdit)
 
-        #Get the list of recently opened files
+        # get the list of recently opened files
         self.populateRecentFiles()
         self.setupWindowGeometry()
         self.defineViewWindowMenu()
@@ -244,8 +248,7 @@ class MainWindow(qt.QMainWindow):
 
     def dragEnterEvent(self, event):
         """Check whether event is valid to be dropped."""
-        if (event.mimeData().hasUrls() and
-            self._getVeuszDropFiles(event)):
+        if event.mimeData().hasUrls() and self._getVeuszDropFiles(event):
             event.acceptProposedAction()
 
     def dropEvent(self, event):
@@ -302,7 +305,7 @@ class MainWindow(qt.QMainWindow):
                 qt.QMessageBox.warning(
                     self, _("Error - Veusz"),
                     _("Unable to load default stylesheet '%s'\n\n%s") %
-                    (filename, cstrerror(e)))
+                    (filename, e.strerror))
             else:
                 # reset any modified flag
                 self.document.setModified(False)
@@ -319,7 +322,7 @@ class MainWindow(qt.QMainWindow):
                 qt.QMessageBox.warning(
                     self, _("Error - Veusz"),
                     _("Unable to load custom definitions '%s'\n\n%s") %
-                    (filename, cstrerror(e)))
+                    (filename, e.strerror))
             else:
                 # reset any modified flag
                 self.document.setModified(False)
@@ -467,6 +470,9 @@ class MainWindow(qt.QMainWindow):
                 a(self, _('Save the current document under a new name'),
                   _('Save &As...'), self.slotFileSaveAs,
                   icon='kde-document-save-as'),
+            'file.trust':
+                a(self, _('Trust document contents'), _('Trust...'),
+                  self.slotFileTrust),
             'file.print':
                 a(self, _('Print the document'), _('&Print...'),
                   self.slotFilePrint,
@@ -582,7 +588,7 @@ class MainWindow(qt.QMainWindow):
             'help.about':
                 a(self, _('Displays information about the program'), _('About...'),
                   self.slotHelpAbout, icon='veusz')
-            }
+        }
 
         # create main toolbar
         tb = self.maintoolbar = qt.QToolBar(_("Main toolbar - Veusz"), self)
@@ -593,8 +599,8 @@ class MainWindow(qt.QMainWindow):
 
         utils.makeMenuGroupSaved(
             'file.new.menu', self, self.vzactions, (
-                    'file.new.graph', 'file.new.graph3d',
-                    'file.new.polar', 'file.new.ternary',
+                'file.new.graph', 'file.new.graph3d',
+                'file.new.polar', 'file.new.ternary',
             )
         )
 
@@ -612,23 +618,30 @@ class MainWindow(qt.QMainWindow):
             tb, self.vzactions,
             ('data.import', 'data.edit',
              'data.create', 'data.capture',
-             'data.filter', 'data.reload'))
+             'data.filter', 'data.reload')
+        )
 
         # menu structure
         filemenu = [
-            ['file.new', _('New'),
-             ['file.new.graph', 'file.new.graph3d', 'file.new.polar',
-              'file.new.ternary']],
+            [
+                'file.new', _('New'),
+                [
+                    'file.new.graph', 'file.new.graph3d', 'file.new.polar',
+                    'file.new.ternary'
+                ]
+            ],
             'file.open',
-            ['file.filerecent', _('Open &Recent'), []],
+            [
+                'file.filerecent', _('Open &Recent'), []
+            ],
             'file.reload',
             '',
-            'file.save', 'file.saveas',
+            'file.save', 'file.saveas', 'file.trust',
             '',
             'file.print', 'file.export',
             '',
             'file.close', 'file.quit'
-            ]
+        ]
         editmenu = [
             'edit.undo', 'edit.redo',
             '',
@@ -636,44 +649,53 @@ class MainWindow(qt.QMainWindow):
             '',
             'edit.prefs', 'edit.stylesheet', 'edit.custom',
             ''
-            ]
+        ]
         viewwindowsmenu = [
             'view.edit', 'view.props', 'view.format',
             'view.console', 'view.datanav',
             '',
             'view.maintool', 'view.viewtool',
             'view.addtool', 'view.edittool'
-            ]
+        ]
         viewmenu = [
-            ['view.viewwindows', _('&Windows'), viewwindowsmenu],
+            [
+                'view.viewwindows', _('&Windows'), viewwindowsmenu
+            ],
             ''
-            ]
+        ]
         insertmenu = [
-            ]
+        ]
 
         # load dataset plugins and create menu
-        datapluginsmenu = self.definePlugins( plugins.datasetpluginregistry,
-                                              self.vzactions, 'data.ops' )
+        datapluginsmenu = self.definePlugins(
+            plugins.datasetpluginregistry,
+            self.vzactions, 'data.ops'
+        )
 
         datamenu = [
-            ['data.ops', _('&Operations'), datapluginsmenu],
+            [
+                'data.ops', _('&Operations'), datapluginsmenu
+            ],
             'data.import', 'data.edit', 'data.create',
             'data.create2d', 'data.capture', 'data.filter', 'data.histogram',
             'data.reload',
-            ]
+        ]
         helpmenu = [
             'help.home', 'help.bug', 'help.update',
             '',
             'help.tutorial',
             '',
-            ['help.examples', _('&Example documents'), []],
+            [
+                'help.examples', _('&Example documents'), []
+            ],
             '',
             'help.about'
-            ]
+        ]
 
         # load tools plugins and create menu
-        toolsmenu = self.definePlugins( plugins.toolspluginregistry,
-                                        self.vzactions, 'tools' )
+        toolsmenu = self.definePlugins(
+            plugins.toolspluginregistry,
+            self.vzactions, 'tools')
 
         menus = [
             ['file', _('&File'), filemenu],
@@ -683,7 +705,7 @@ class MainWindow(qt.QMainWindow):
             ['data', _('&Data'), datamenu],
             ['tools', _('&Tools'), toolsmenu],
             ['help', _('&Help'), helpmenu],
-            ]
+        ]
 
         self.menus = {}
         utils.constructMenus(self.menuBar(), self.menus, menus, self.vzactions)
@@ -704,9 +726,11 @@ class MainWindow(qt.QMainWindow):
 
         # not cstr here forces to unicode for Python 2, getting
         # filenames in unicode
-        examples = [ os.path.join(utils.exampleDirectory, f)
-                     for f in os.listdir(cstr(utils.exampleDirectory))
-                     if os.path.splitext(f)[1] == ".vsz" ]
+        examples = [
+            os.path.join(utils.exampleDirectory, f)
+            for f in os.listdir(str(utils.exampleDirectory))
+            if os.path.splitext(f)[1] == ".vsz"
+        ]
 
         menu = self.menus["help.examples"]
         for ex in sorted(examples):
@@ -730,16 +754,18 @@ class MainWindow(qt.QMainWindow):
 
         # set whether windows are visible and connect up to toggle windows
         self.viewwinfns = []
-        for win, act in ((self.treeedit, 'view.edit'),
-                         (self.propdock, 'view.props'),
-                         (self.formatdock, 'view.format'),
-                         (self.console, 'view.console'),
-                         (self.datadock, 'view.datanav'),
-                         (self.maintoolbar, 'view.maintool'),
-                         (self.datatoolbar, 'view.datatool'),
-                         (self.treeedit.edittoolbar, 'view.edittool'),
-                         (self.treeedit.addtoolbar, 'view.addtool'),
-                         (self.plot.viewtoolbar, 'view.viewtool')):
+        for win, act in (
+                (self.treeedit, 'view.edit'),
+                (self.propdock, 'view.props'),
+                (self.formatdock, 'view.format'),
+                (self.console, 'view.console'),
+                (self.datadock, 'view.datanav'),
+                (self.maintoolbar, 'view.maintool'),
+                (self.datatoolbar, 'view.datatool'),
+                (self.treeedit.edittoolbar, 'view.edittool'),
+                (self.treeedit.addtoolbar, 'view.addtool'),
+                (self.plot.viewtoolbar, 'view.viewtool')
+        ):
 
             a = self.vzactions[act]
             fn = viewHideWindow(win)
@@ -849,7 +875,7 @@ class MainWindow(qt.QMainWindow):
               "Would you like to start the tutorial now?\n"
               "If not, you can access it later through the Help menu."),
             qt.QMessageBox.Yes | qt.QMessageBox.No
-            )
+        )
 
         if retn == qt.QMessageBox.Yes:
             self.slotHelpTutorial()
@@ -883,10 +909,10 @@ class MainWindow(qt.QMainWindow):
         dayssinceinstall = (
             datetime.date.today() -
             datetime.date(*setting.settingdb['install_date'])).days
-        if (dayssinceinstall<mininterval or
-            setting.settingdb['vercheck_asked_user'] or
-            setting.settingdb['vercheck_disabled'] or
-            utils.disableVersionChecks):
+        if ( dayssinceinstall<mininterval or
+             setting.settingdb['vercheck_asked_user'] or
+             setting.settingdb['vercheck_disabled'] or
+             utils.disableVersionChecks ):
             return
 
         retn = qt.QMessageBox.question(
@@ -896,7 +922,7 @@ class MainWindow(qt.QMainWindow):
               "Is this ok? This choice can be changed in Preferences."),
             qt.QMessageBox.Yes | qt.QMessageBox.No,
             qt.QMessageBox.Yes
-            )
+        )
 
         setting.settingdb['vercheck_disabled'] = retn==qt.QMessageBox.No
         setting.settingdb['vercheck_asked_user'] = True
@@ -923,10 +949,10 @@ class MainWindow(qt.QMainWindow):
         dayssinceinstall = (
             datetime.date.today() -
             datetime.date(*setting.settingdb['install_date'])).days
-        if (dayssinceinstall<mininterval or
-            setting.settingdb['feedback_asked_user'] or
-            setting.settingdb['feedback_disabled'] or
-            utils.disableFeedback):
+        if ( dayssinceinstall<mininterval or
+             setting.settingdb['feedback_asked_user'] or
+             setting.settingdb['feedback_disabled'] or
+             utils.disableFeedback ):
             return
 
         retn = qt.QMessageBox.question(
@@ -938,7 +964,7 @@ class MainWindow(qt.QMainWindow):
               "Is this ok? This choice can be changed in Preferences."),
             qt.QMessageBox.Yes | qt.QMessageBox.No,
             qt.QMessageBox.Yes
-            )
+        )
 
         setting.settingdb['feedback_disabled'] = retn==qt.QMessageBox.No
         setting.settingdb['feedback_asked_user'] = True
@@ -976,7 +1002,8 @@ class MainWindow(qt.QMainWindow):
             _("Save file?"),
             _("Document%s was modified. Save first?") % filetext,
             qt.QMessageBox.Save | qt.QMessageBox.Discard |
-            qt.QMessageBox.Cancel)
+            qt.QMessageBox.Cancel
+        )
 
     def closeEvent(self, event):
         """Before closing, check whether we need to save first."""
@@ -999,7 +1026,7 @@ class MainWindow(qt.QMainWindow):
 
         # store docked windows
         data = self.saveState().data()
-        setdb['geometry_mainwindowstate'] = cbytes(data)
+        setdb['geometry_mainwindowstate'] = bytes(data)
 
         # save current setting db
         setdb.writeSettings()
@@ -1055,34 +1082,29 @@ class MainWindow(qt.QMainWindow):
         if self.filename == '':
             self.slotFileSaveAs()
         else:
-            # show busy cursor
-            qt.QApplication.setOverrideCursor( qt.QCursor(qt.Qt.WaitCursor) )
             try:
-                ext = os.path.splitext(self.filename)[1]
-                mode = 'hdf5' if ext == '.vszh5' else 'vsz'
-                self.document.save(self.filename, mode)
-                self.updateStatusbar(_("Saved to %s") % self.filename)
+                with utils.OverrideCursor():
+                    ext = os.path.splitext(self.filename)[1]
+                    mode = 'hdf5' if ext == '.vszh5' else 'vsz'
+                    self.document.save(self.filename, mode)
+                    self.updateStatusbar(_("Saved to %s") % self.filename)
             except EnvironmentError as e:
-                qt.QApplication.restoreOverrideCursor()
                 qt.QMessageBox.critical(
                     self, _("Error - Veusz"),
                     _("Unable to save document as '%s'\n\n%s") %
-                    (self.filename, cstrerror(e)))
-            else:
-                # restore the cursor
-                qt.QApplication.restoreOverrideCursor()
+                    (self.filename, e.strerror))
 
     def updateTitlebar(self):
         """Put the filename into the title bar."""
         if self.filename == '':
             self.setWindowTitle(_('Untitled - Veusz'))
         else:
-            self.setWindowTitle( _("%s - Veusz") %
-                                 os.path.basename(self.filename) )
+            self.setWindowTitle(
+                _("%s - Veusz") % os.path.basename(self.filename))
 
     def plotQueueChanged(self, incr):
         self.plotqueuecount += incr
-        text = u'•' * self.plotqueuecount
+        text = '•' * self.plotqueuecount
         self.plotqueuelabel.setText(text)
 
     def fileSaveDialog(self, filters, dialogtitle):
@@ -1145,7 +1167,7 @@ class MainWindow(qt.QMainWindow):
                 qt.QMessageBox.critical(
                     self, _("Error - Veusz"),
                     _("Unable to open '%s'\n\n%s") %
-                    (filename, cstrerror(e)))
+                    (filename, e.strerror))
                 return None
             return filename
         return None
@@ -1183,39 +1205,24 @@ class MainWindow(qt.QMainWindow):
         def _callbackunsafe():
             """Callback when loading document to ask whether ok to continue loading
             if unsafe commands are found."""
-            qt.QApplication.restoreOverrideCursor()
-            msgbox = qt.QMessageBox(
-                qt.QMessageBox.Warning,
-                _("Unsafe code in document"),
-                _("The document '%s' contains potentially unsafe code "
-                  "which may damage your computer or data. Please check "
-                  "that the file comes from a trusted source.") % filename,
-                qt.QMessageBox.NoButton,
-                self)
-            cont = msgbox.addButton(_("C&ontinue anyway"), qt.QMessageBox.AcceptRole)
-            stop = msgbox.addButton(_("&Stop loading"), qt.QMessageBox.RejectRole)
-            msgbox.setDefaultButton(stop)
-            msgbox.exec_()
-            qt.QApplication.setOverrideCursor(qt.QCursor(qt.Qt.WaitCursor))
-            return msgbox.clickedButton() is cont
+            return self.checkUnsafe()
 
         def _callbackimporterror(filename, error):
             """Ask user if they want to give a new filename in case of import
             error.
             """
-            qt.QApplication.restoreOverrideCursor()
-            msgbox = qt.QMessageBox(self)
-            msgbox.setWindowTitle(_("Import error"))
-            msgbox.setText(
-                _("Could not import data from file '%s':\n\n %s") % (
-                    filename, error))
-            msgbox.setInformativeText(_("Do you want to look for another file?"))
-            msgbox.setStandardButtons(qt.QMessageBox.Yes | qt.QMessageBox.Cancel)
-            filename = None
-            if msgbox.exec_() == qt.QMessageBox.Yes:
-                filename = qt.QFileDialog.getOpenFileName(self, "Choose data file")
-                filename = filename[0] if filename else None
-            qt.QApplication.setOverrideCursor(qt.QCursor(qt.Qt.WaitCursor))
+            with utils.OverrideCursor(qt.Qt.ArrowCursor):
+                msgbox = qt.QMessageBox(self)
+                msgbox.setWindowTitle(_("Import error"))
+                msgbox.setText(
+                    _("Could not import data from file '%s':\n\n %s") % (
+                        filename, error))
+                msgbox.setInformativeText(_("Do you want to look for another file?"))
+                msgbox.setStandardButtons(qt.QMessageBox.Yes | qt.QMessageBox.Cancel)
+                filename = None
+                if msgbox.exec_() == qt.QMessageBox.Yes:
+                    filename = qt.QFileDialog.getOpenFileName(self, "Choose data file")
+                    filename = filename[0] if filename else None
             return filename
 
         # save stdout and stderr, then redirect to console
@@ -1223,8 +1230,6 @@ class MainWindow(qt.QMainWindow):
         sys.stdout = self.console.con_stdout
         sys.stderr = self.console.con_stderr
 
-        qt.QApplication.setOverrideCursor( qt.QCursor(qt.Qt.WaitCursor) )
-
         try:
             # get loading mode
             ext = os.path.splitext(filename)[1].lower()
@@ -1236,27 +1241,25 @@ class MainWindow(qt.QMainWindow):
                 raise document.LoadError(
                     _("Did not recognise file type '%s'") % ext)
 
-            # do the actual loading
-            self.document.load(
-                filename,
-                mode=mode,
-                callbackunsafe=_callbackunsafe,
-                callbackimporterror=_callbackimporterror)
+            with utils.OverrideCursor():
+                # do the actual loading
+                self.document.load(
+                    filename,
+                    mode=mode,
+                    callbackunsafe=_callbackunsafe,
+                    callbackimporterror=_callbackimporterror)
 
         except document.LoadError as e:
             from ..dialogs.errorloading import ErrorLoadingDialog
-            qt.QApplication.restoreOverrideCursor()
             if e.backtrace:
-                d = ErrorLoadingDialog(self, filename, cstr(e), e.backtrace)
+                d = ErrorLoadingDialog(self, filename, str(e), e.backtrace)
                 d.exec_()
             else:
                 qt.QMessageBox.critical(
                     self, _("Error opening %s - Veusz") % filename,
-                    cstr(e))
+                    str(e))
             return False
 
-        qt.QApplication.restoreOverrideCursor()
-
         # need to remember to restore stdout, stderr
         sys.stdout, sys.stderr = stdout, stderr
 
@@ -1324,8 +1327,8 @@ class MainWindow(qt.QMainWindow):
         menu.clear()
 
         if setdb['main_recentfiles']:
-            files = [f for f in setdb['main_recentfiles']
-                     if os.path.isfile(f)]
+            files = [
+                f for f in setdb['main_recentfiles'] if os.path.isfile(f)]
 
             # add each recent file to menu
             newmenuitems = []
@@ -1350,7 +1353,8 @@ class MainWindow(qt.QMainWindow):
             _("Reload file"),
             _("Reload document from file, losing any changes?"),
             qt.QMessageBox.Yes | qt.QMessageBox.Cancel,
-            qt.QMessageBox.Cancel)
+            qt.QMessageBox.Cancel
+        )
         if retn == qt.QMessageBox.Yes:
             if not os.path.exists(self.filename):
                 qt.QMessageBox.critical(
@@ -1392,11 +1396,11 @@ class MainWindow(qt.QMainWindow):
     def slotUpdatePage(self, number):
         """Update page number when the plot window says so."""
 
-        np = self.document.getNumberPages()
-        if np == 0:
+        nump = self.document.getNumberPages()
+        if nump == 0:
             self.pagelabel.setText(_("No pages"))
         else:
-            self.pagelabel.setText(_("Page %i/%i") % (number+1, np))
+            self.pagelabel.setText(_("Page %i/%i") % (number+1, nump))
 
     def slotUpdateAxisValues(self, values):
         """Update the position where the mouse is relative to the axes."""
@@ -1431,7 +1435,8 @@ class MainWindow(qt.QMainWindow):
             if dtype == 'date':
                 return utils.dateFloatToString(val)
             elif dtype == 'numeric':
-                return '%0.5g' % val
+                fmt = '%.'+str(setting.settingdb['picker_sig_figs'])+'g'
+                return fmt % val
             elif dtype == 'text':
                 return val
             else:
@@ -1452,8 +1457,97 @@ class MainWindow(qt.QMainWindow):
             else:
                 qt.QApplication.clipboard().setText(t)
 
-    def slotAllowedImportsDoc(self, module, names):
+    def checkUnsafe(self):
+        """Ask user if code should be unsafe."""
+
+        # we shouldn't allow these places to be added to allowed safe directories
+        badlocs = (
+            qt.QStandardPaths.standardLocations(qt.QStandardPaths.DownloadLocation) +
+            qt.QStandardPaths.standardLocations(qt.QStandardPaths.TempLocation)
+        )
+        fname = self.document.filename
+        absfile = os.path.abspath(fname)
+        filedir = '' if not fname else os.path.dirname(absfile)
+        isbadloc = False
+        if not filedir:
+            # shouldn't get here, but don't allow empty dir to be added
+            isbadloc = True
+        for path in badlocs:
+            if absfile.startswith(os.path.abspath(path) + os.sep):
+                isbadloc = True
+        # don't allow plain home directory
+        if filedir == qt.QStandardPaths.standardLocations(
+                qt.QStandardPaths.HomeLocation)[0]:
+            isbadloc = True
+
+        msgbox = qt.QMessageBox(
+            qt.QMessageBox.Warning,
+            _("Potentially unsafe code in document"),
+            _(
+                "<p><b>The document '%s' contains potentially unsafe code</b></p>"
+                "<p>Directory: '%s'</p>"
+                "<p>This file could damage your computer or data "
+                "as it can contain arbitrary code. "
+                "Please check "
+                "that the file was made by you or a trusted source.</p>"
+            ) % (
+                os.path.basename(absfile) if fname else "",
+                filedir if fname else "",
+            ),
+            qt.QMessageBox.NoButton,
+            self,
+        )
+        allow = msgbox.addButton(_("Allow"), qt.QMessageBox.AcceptRole)
+        addloc = msgbox.addButton(_("Add to trusted locations"), qt.QMessageBox.AcceptRole)
+        addloc.setEnabled(not isbadloc)
+        stop = msgbox.addButton(_("Skip"), qt.QMessageBox.RejectRole)
+
+        msgbox.setDefaultButton(stop)
+
+        # we enter here with a busy cursor often, so set back to arrow temporarily
+        with utils.OverrideCursor(qt.Qt.ArrowCursor):
+            msgbox.exec_()
+
+        clicked = msgbox.clickedButton()
+        if clicked is addloc:
+            with utils.OverrideCursor(qt.Qt.ArrowCursor):
+                button = qt.QMessageBox.warning(
+                    self, _("Are you sure?"),
+                    _("Are you really sure that you want to add directory '%s' to the "
+                      "list of trusted locations. Any file loaded from this directory "
+                      "will be trusted.") % filedir,
+                    qt.QMessageBox.Yes | qt.QMessageBox.No,
+                    qt.QMessageBox.No,
+                )
+            if button == qt.QMessageBox.Yes:
+                setting.settingdb['secure_dirs'].append(filedir)
+                return True
+            else:
+                return False
+
+        return clicked is allow
+
+    def slotUpdateSecurity(self, secure):
+        """Show or hide security label and trust menu based on security"""
+        self.securitylabel.setVisible(not secure)
+        self.vzactions['file.trust'].setVisible(not secure)
+
+    def slotAllowedImportsDoc(self):
         """Are allowed imports?"""
-        from ..dialogs.safetyimport import SafetyImportDialog
-        d = SafetyImportDialog(self, module, names)
-        d.exec_()
+        if self.checkUnsafe():
+            self.document.evaluate.setSecurity(True)
+
+    def slotFileTrust(self):
+        """User requests that document should be trusted."""
+        button = qt.QMessageBox.warning(
+            self, _("Are you sure?"),
+            _("Are you sure that you want to trust the document contents, "
+              "including any potentially dangerous code? Only trust "
+              "documents with a trusted source."),
+            qt.QMessageBox.Yes | qt.QMessageBox.No,
+            qt.QMessageBox.No,
+        )
+        if button == qt.QMessageBox.Yes:
+            self.document.evaluate.setSecurity(True)
+            # force redraw of document
+            self.plot.actionForceUpdate()
